/**
 * OW2 Util
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:WarArchiveMetadata.java 2372 2008-02-08 18:18:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.war.impl.configurator;

import org.ow2.util.ee.metadata.war.api.IWarClassMetadata;
import org.ow2.util.ee.metadata.war.api.IWarFieldMetadata;
import org.ow2.util.ee.metadata.war.api.IWarMethodMetadata;
import org.ow2.util.ee.metadata.war.impl.WarClassMetadata;
import org.ow2.util.ee.metadata.war.impl.WarDeployableMetadata;
import org.ow2.util.ee.metadata.war.impl.specific.ISpecificWarDeployableMetadataConfigurator;
import org.ow2.util.ee.metadata.ws.impl.configurator.WebservicesClassMetadataConfigurator;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

import java.util.List;
import java.util.Set;

/**
 * WarClassMetadataConfigurator.
 * @author Gael Lalire
 */
public class WarClassMetadataConfigurator extends
        WebservicesClassMetadataConfigurator<IWarClassMetadata, IWarMethodMetadata, IWarFieldMetadata> {

    /**
     * annotationParsingDesactived.
     */
    private boolean annotationParsingDesactived;

    /**
     * WarClassMetadataConfigurator.
     * @param jClass signature
     * @param warArchiveMetadata parent
     * @param annotationParsingDesactived annotationParsingDesactived
     * @param specificWarDeployableMetadataConfiguratorList specificWarDeployableMetadataConfiguratorList
     */
    public WarClassMetadataConfigurator(
            final JClass jClass,
            final WarDeployableMetadata warArchiveMetadata,
            final boolean annotationParsingDesactived,
            final List<ISpecificWarDeployableMetadataConfigurator>
                specificWarDeployableMetadataConfiguratorList) {
        super(new WarClassMetadata(jClass, warArchiveMetadata), specificWarDeployableMetadataConfiguratorList);
        this.annotationParsingDesactived = annotationParsingDesactived;
        if (!annotationParsingDesactived) {
            registerAnnotationVisitor();
        }
    }

    /**
     * @param jField signature
     * @return configurator
     */
    @Override
    protected IFieldConfigurator createFieldMetadataConfigurator(final JField jField) {
        return new WarFieldMetadataConfigurator(jField, getClassMetadata(), annotationParsingDesactived,
                getSpecificClassConfiguratorList());
    }

    /**
     * @param jMethod signature
     * @return configurator
     */
    @Override
    protected IMethodConfigurator createMethodMetadataConfigurator(final JMethod jMethod) {
        return new WarMethodMetadataConfigurator(jMethod, getClassMetadata(), annotationParsingDesactived,
                getSpecificClassConfiguratorList());
    }

    /**
     * add class to archive.
     * @param referencedClasses you may fill this set with class referenced in current metadata
     */
    @Override
    public void configurationComplete(final Set<String> referencedClasses) {
        IWarClassMetadata warClassMetadata = getClassMetadata();
        warClassMetadata.getWarDeployableMetadata().addWarClassMetadata(warClassMetadata);
        super.configurationComplete(referencedClasses);
    }

}
